@@ -12,10 +12,14 @@ from discord_system_observer_bot.statsobserver import collect_stats as _collect_
 from discord_system_observer_bot.statsobserver import stats2rows
 from discord_system_observer_bot.statsobserver import plot_rows
 from discord_system_observer_bot.statsobserver import has_extra_deps_gpu
-from discord_system_observer_bot.statsobserver import make_observable_limits
-from discord_system_observer_bot.statsobserver import NotifyBadCounterManager
+from discord_system_observer_bot.statsobserver import (
+    make_observable_limits,
+    LimitTypesSetType,
+    NotifyBadCounterManager,
+)
 from discord_system_observer_bot.sysinfo import get_local_machine_name
 from discord_system_observer_bot.sysinfo import get_cpu_info, get_disk_info
+from discord_system_observer_bot.utils import make_table, dump_dict_kv
 
 
 LOGGER = logging.getLogger(__name__)
@@ -24,8 +28,52 @@ LOGGER = logging.getLogger(__name__)
 # ---------------------------------------------------------------------------
 
 
-def make_sysinfo_message(cpu=True, disk=True, gpu=True):
-    message = f"**Status of `{get_local_machine_name()}`**\n"
+# Hack, will not allow multiple instances to be run in threads
+# Well, should previously have been the same, as the local_machine_name
+#   is always the same.
+
+
+_LOCAL_MACHINE_NAME = get_local_machine_name()
+
+
+def get_name() -> str:
+    """Gets the locally stored "machine" name.
+
+    Returns
+    -------
+    str
+        name of the machine the bot runs on
+    """
+    return _LOCAL_MACHINE_NAME
+
+
+def set_name(name: str) -> None:
+    """Set the local "machine" name.
+
+    Parameters
+    ----------
+    name : str
+        the name of the machine/system the bot runs on
+    """
+    global _LOCAL_MACHINE_NAME  # pylint: disable=global-statement
+    # NOTE: using global here is the easiest solution
+    #       compared to creating a class for storing and accessing
+    #       a single variable ...
+    _LOCAL_MACHINE_NAME = name
+
+
+# ---------------------------------------------------------------------------
+
+
+def make_sysinfo_message(
+    cpu: bool = True,
+    disk: bool = True,
+    gpu: bool = True,
+    name: typing.Optional[str] = None,
+) -> str:
+    if name is None:
+        name = get_name()
+    message = f"**Status of `{name}`**\n"
     message += f"Date: `{datetime.datetime.now()}`\n\n"
 
     if cpu:
@@ -55,8 +103,15 @@ def make_sysinfo_message(cpu=True, disk=True, gpu=True):
     return message
 
 
-def make_sysinfo_embed(cpu=True, disk=True, gpu=True):
-    embed = discord.Embed(title=f"System Status of `{get_local_machine_name()}`")
+def make_sysinfo_embed(
+    cpu: bool = True,
+    disk: bool = True,
+    gpu: bool = True,
+    name: typing.Optional[str] = None,
+) -> discord.Embed:
+    if name is None:
+        name = get_name()
+    embed = discord.Embed(title=f"System Status of `{name}`")
 
     # embed.set_thumbnail(url="")  # TODO: add "private" logo (maybe as an config option ...)
 
@@ -86,17 +141,19 @@ class SelfOrAllName:
     name is star '*' or the actual name. If not raise BadArgument
     to abort subcommand execution."""
 
-    def __init__(self, name):
+    def __init__(self, name: str):
         self._name = name
 
     @classmethod
-    async def convert(cls, ctx, argument):  # pylint: disable=unused-argument
-        if argument not in ("*", get_local_machine_name()):
+    async def convert(
+        cls, ctx, argument: str  # pylint: disable=unused-argument
+    ) -> "SelfOrAllName":
+        if argument not in ("*", get_name()):
             raise commands.BadArgument("Not the local machine name or wildcard!")
         return cls(argument)
 
     @property
-    def name(self):
+    def name(self) -> str:
         return self._name
 
     def __str__(self):
@@ -107,20 +164,18 @@ class SelfOrAllName:
 
 
 class SystemResourceObserverCog(commands.Cog, name="System Resource Observer"):
-    def __init__(self, bot, channel_id):
+    def __init__(self, bot: "ObserverBot", limits_types: LimitTypesSetType = None):
         self.bot = bot
-        self.channel_id = channel_id
-        self.local_machine_name = get_local_machine_name()
 
         self.limits = dict()
         self.bad_checker = NotifyBadCounterManager()
         self.stats = defaultdict(int)
 
-        self.init_limits()
+        self.init_limits(limits_types=limits_types)
 
-    def init_limits(self):
+    def init_limits(self, limits_types: LimitTypesSetType = None):
         # TODO: pack them in an optional file (like Flask configs) and try to load else nothing.
-        self.limits.update(make_observable_limits())
+        self.limits.update(make_observable_limits(include=limits_types))
 
     def reset_notifications(self):
         self.bad_checker.reset()
@@ -129,7 +184,7 @@ class SystemResourceObserverCog(commands.Cog, name="System Resource Observer"):
     async def observe_system(self):
         LOGGER.debug("Running observe system task loop ...")
 
-        async with self.bot.get_channel(self.channel_id).typing():
+        async with self.bot.get_channel(self.bot.channel_id).typing():
             # perform checks
             for name, limit in self.limits.items():
                 try:
@@ -158,8 +213,10 @@ class SystemResourceObserverCog(commands.Cog, name="System Resource Observer"):
                 # check if already notified (that limit reached)
                 # even if shortly recovered but not completely, e. g. 3->2->3 >= 3 (thres) <= 0 (not completely reset)
                 await self.send(
-                    limit.message.format(cur_value=cur_value, threshold=limit.threshold)
-                    + f" @`{self.local_machine_name}`"
+                    limit.message.format(
+                        cur_value=cur_value, threshold=limit.threshold, unit=limit.unit
+                    )
+                    + f" @`{self.bot.local_machine_name}`"
                 )
                 self.bad_checker.mark_notified(name)
                 self.stats["num_limits_notified"] += 1
@@ -167,7 +224,7 @@ class SystemResourceObserverCog(commands.Cog, name="System Resource Observer"):
             if self.bad_checker.decrease_counter(name):
                 # get one-time True if changed from non-normal to normal
                 await self.send(
-                    f"*{limit.name} has recovered*" f" @`{self.local_machine_name}`"
+                    f"*{limit.name} has recovered*" f" @`{self.bot.local_machine_name}`"
                 )
                 self.stats["num_normal_notified"] += 1
 
@@ -178,7 +235,7 @@ class SystemResourceObserverCog(commands.Cog, name="System Resource Observer"):
 
     async def send(self, message):
         # TODO: send to default channel?
-        channel = self.bot.get_channel(self.channel_id)
+        channel = self.bot.get_channel(self.bot.channel_id)
         await channel.send(message)
 
     def cog_unload(self):
@@ -213,10 +270,10 @@ class SystemResourceObserverCog(commands.Cog, name="System Resource Observer"):
         # NOTE: check for is_running() only added in version 1.4.0
         if self.observe_system.get_task() is None:  # pylint: disable=no-member
             self.observe_system.start()  # pylint: disable=no-member
-            await ctx.send(f"Observer started @`{self.local_machine_name}`")
+            await ctx.send(f"Observer started @`{self.bot.local_machine_name}`")
         else:
             self.observe_system.restart()  # pylint: disable=no-member
-            await ctx.send(f"Observer restarted @`{self.local_machine_name}`")
+            await ctx.send(f"Observer restarted @`{self.bot.local_machine_name}`")
 
     @observer_cmd.command(name="stop")
     @commands.cooldown(1.0, 10.0)
@@ -224,7 +281,14 @@ class SystemResourceObserverCog(commands.Cog, name="System Resource Observer"):
         """Stops the background system observer."""
         self.observe_system.cancel()  # pylint: disable=no-member
         self.reset_notifications()
-        await ctx.send(f"Observer stopped @`{self.local_machine_name}`")
+        await ctx.send(f"Observer stopped @`{self.bot.local_machine_name}`")
+
+    def _header_for(self, name: str) -> str:
+        return (
+            f"**{name} for** `{self.bot.local_machine_name}`"  # pylint: disable=no-member
+            f""" [`{"running" if self.observe_system.next_iteration is not None else "stopped"}`]"""
+            "\n"
+        )
 
     @observer_cmd.command(name="status")
     @commands.cooldown(1.0, 10.0)
@@ -232,16 +296,9 @@ class SystemResourceObserverCog(commands.Cog, name="System Resource Observer"):
         """Displays statistics about notifications etc."""
 
         if not self.stats:
-            await ctx.send(f"N/A [`{self.local_machine_name}`] [`not-started`]")
+            await ctx.send(f"N/A [`{self.bot.local_machine_name}`] [`not-started`]")
             return
 
-        len_keys = max(len(k) for k in self.stats.keys())
-        len_vals = max(
-            len(str(v))
-            for v in self.stats.values()
-            if isinstance(v, (int, float, bool))
-        )
-
         try:
             # pylint: disable=no-member
             next_time = self.observe_system.next_iteration - datetime.datetime.now(
@@ -254,13 +311,8 @@ class SystemResourceObserverCog(commands.Cog, name="System Resource Observer"):
 
         message = "".join(
             [
-                f"**Observer status for** `{self.local_machine_name}`",
-                f""" [`{"running" if self.observe_system.next_iteration is not None else "stopped"}`]""",  # pylint: disable=no-member
-                "\n```\n",
-                "\n".join(
-                    [f"{k:<{len_keys}} {v:>{len_vals}}" for k, v in self.stats.items()]
-                ),
-                "\n```",
+                self._header_for("Observer status"),
+                dump_dict_kv(self.stats, wrap_markdown=True),
                 f"\nNext check in `{next_time}`",
             ]
         )
@@ -273,28 +325,31 @@ class SystemResourceObserverCog(commands.Cog, name="System Resource Observer"):
         """Dump current badness values."""
 
         if not self.bad_checker.bad_counters:
-            await ctx.send(f"N/A [`{self.local_machine_name}`] [`not-started`]")
+            await ctx.send(f"N/A [`{self.bot.local_machine_name}`] [`not-started`]")
             return
 
-        len_keys = max(len(k) for k in self.bad_checker.bad_counters.keys())
-        len_vals = max(
-            len(str(v))
-            for v in self.bad_checker.bad_counters.values()
-            if isinstance(v, (int, float, bool))
-        )
-
         message = "".join(
             [
-                f"**Badness values for** `{self.local_machine_name}`",
-                f""" [`{"running" if self.observe_system.next_iteration is not None else "stopped"}`]""",  # pylint: disable=no-member
-                "\n```\n",
-                "\n".join(
+                self._header_for("Badness values"),
+                # dump_dict_kv(self.bad_checker.bad_counters, wrap_markdown=True),
+                make_table(
                     [
-                        f"{k:<{len_keys}} {v:>{len_vals}}"
-                        for k, v in self.bad_checker.bad_counters.items()
-                    ]
+                        (
+                            v.name,
+                            self.bad_checker.bad_counters[k],
+                            v.badness_inc,
+                            v.badness_dec,
+                            v.badness_threshold,
+                            self.bad_checker.notified[k],
+                        )
+                        for k, v in self.limits.items()
+                    ],
+                    ("name", "badness", "inc", "dec", "max", "notified"),
+                    alignments=("<", ">", ">", ">", ">", ">"),
+                    wrap_markdown=True,
+                    header_separator=True,
+                    column_separators=False,
                 ),
-                "\n```",
             ]
         )
 
@@ -305,24 +360,34 @@ class SystemResourceObserverCog(commands.Cog, name="System Resource Observer"):
     async def observer_dump_limits(self, ctx):
         """Write out limits."""
 
-        len_keys = max(len(k) for k in self.limits.keys())
-        len_vals = 10
+        def _get_safe_current(limit):
+            try:
+                return limit.fn_retrieve()
+            except:  # pylint: disable=bare-except
+                return None
 
         message = "".join(
             [
-                f"**Limits for** `{self.local_machine_name}`",
-                f""" [`{"running" if self.observe_system.next_iteration is not None else "stopped"}`]""",  # pylint: disable=no-member
-                "\n```\n",
-                f"""{"name":<{len_keys}} {"threshold":>{len_vals}} {"exceed?":>{len_vals}} {"notified":>{len_vals}}\n""",
-                "\n".join(
+                self._header_for("Limits"),
+                make_table(
                     [
-                        f"{k:<{len_keys}} {v.threshold:>{len_vals}} "
-                        f"{str(self.bad_checker.threshold_reached(k, v)):>{len_vals}} "
-                        f"{str(self.bad_checker.notified[k]):>{len_vals}}"
-                        for k, v in self.limits.items()
-                    ]
+                        (
+                            limit.name,
+                            # lid, "id"
+                            _get_safe_current(limit),
+                            limit.threshold,
+                            limit.unit,
+                            self.bad_checker.threshold_reached(lid, limit),
+                            self.bad_checker.notified[lid],
+                        )
+                        for lid, limit in self.limits.items()
+                    ],
+                    ("name", "current", "threshold", "unit", "exceed?", "notified"),
+                    alignments=("<", ">", ">", "<", ">", ">"),
+                    wrap_markdown=True,
+                    header_separator=True,
+                    column_separators=False,
                 ),
-                "\n```",
             ]
         )
 
@@ -333,10 +398,8 @@ class SystemResourceObserverCog(commands.Cog, name="System Resource Observer"):
 
 
 class SystemStatsCollectorCog(commands.Cog, name="System Statistics Collector"):
-    def __init__(self, bot, channel_id):
+    def __init__(self, bot: "ObserverBot"):
         self.bot = bot
-        self.channel_id = channel_id
-        self.local_machine_name = get_local_machine_name()
 
         # for a total of a week
         #   10 / 60 how often per minute,
@@ -348,7 +411,7 @@ class SystemStatsCollectorCog(commands.Cog, name="System Statistics Collector"):
     async def collect_stats(self):
         LOGGER.debug("Running collect system stats task loop ...")
 
-        async with self.bot.get_channel(self.channel_id).typing():
+        async with self.bot.get_channel(self.bot.channel_id).typing():
             # collect stats
             try:
                 cur_stats = _collect_stats(include=("cpu", "disk", "gpu"))
@@ -380,24 +443,24 @@ class SystemStatsCollectorCog(commands.Cog, name="System Statistics Collector"):
         # NOTE: check for is_running() only added in version 1.4.0
         if self.collect_stats.get_task() is None:  # pylint: disable=no-member
             self.collect_stats.start()  # pylint: disable=no-member
-            await ctx.send(f"Collector started @`{self.local_machine_name}`")
+            await ctx.send(f"Collector started @`{self.bot.local_machine_name}`")
         else:
             self.collect_stats.restart()  # pylint: disable=no-member
-            await ctx.send(f"Collector restarted @`{self.local_machine_name}`")
+            await ctx.send(f"Collector restarted @`{self.bot.local_machine_name}`")
 
     @collector_cmd.command(name="stop")
     @commands.cooldown(1.0, 10.0)
     async def collector_stop(self, ctx):
         """Stops the background system statistics collector."""
         self.collect_stats.cancel()  # pylint: disable=no-member
-        await ctx.send(f"Collector stopped @`{self.local_machine_name}`")
+        await ctx.send(f"Collector stopped @`{self.bot.local_machine_name}`")
 
     @collector_cmd.command(name="plot")
     @commands.cooldown(1.0, 10.0)
     async def collector_plot(self, ctx):
         """Plots collected stats."""
         if not self.stats:
-            await ctx.send(f"N/A @`{self.local_machine_name}`")
+            await ctx.send(f"N/A @`{self.bot.local_machine_name}`")
             return
 
         series = stats2rows(self.stats)
@@ -409,105 +472,61 @@ class SystemStatsCollectorCog(commands.Cog, name="System Statistics Collector"):
             filename=f"plot-{datetime.datetime.now(datetime.timezone.utc)}.png",
         )
 
-        await ctx.send(f"Plot @`{self.local_machine_name}`", file=dfile)
+        await ctx.send(f"Plot @`{self.bot.local_machine_name}`", file=dfile)
 
 
 # ---------------------------------------------------------------------------
 
 
 class GeneralCommandsCog(commands.Cog, name="General"):
-    def __init__(self, bot):
+    def __init__(self, bot: "ObserverBot"):
         self.bot = bot
-        self.local_machine_name = get_local_machine_name()
 
     @commands.command()
     async def ping(self, ctx):
         """Standard Ping-Pong latency/is-alive test."""
         await ctx.send(
-            f"Pong (latency: {self.bot.latency * 1000:.1f} ms) @`{self.local_machine_name}`"
+            f"Pong (latency: {self.bot.latency * 1000:.1f} ms) @`{self.bot.local_machine_name}`"
         )
 
     @commands.command()
     async def info(self, ctx):
         """Query local system information and send it back."""
-        embed = make_sysinfo_embed()
+        embed = make_sysinfo_embed(name=self.bot.local_machine_name)
         await ctx.send(embed=embed)
 
 
 # ---------------------------------------------------------------------------
 
 
-def run_observer_old(token, channel_id):
-    """Functional variant, using decorators."""
-    observer_bot = commands.Bot(command_prefix=".")
-
-    @observer_bot.event
-    async def on_ready():  # pylint: disable=unused-variable
-        LOGGER.info(f"Logged on as {observer_bot.user}")
-        LOGGER.debug(f"name: {observer_bot.user.name}, id: {observer_bot.user.id}")
-
-        if channel_id is not None:
-            channel = observer_bot.get_channel(channel_id)
-            LOGGER.info(f"Channel: {channel} {type(channel)} {repr(channel)}")
-            await channel.send(
-                f"Running observer bot on `{get_local_machine_name()}`...\n"
-                f"Type `{observer_bot.command_prefix}help` to display available commands."
-            )
-
-        await observer_bot.change_presence(status=discord.Status.idle)
-
-        # TODO: maybe start observe_system task here (if required?)
-
-    @observer_bot.event
-    async def on_disconnect():  # pylint: disable=unused-variable
-        LOGGER.warning(f"Bot {observer_bot.user} disconnected!")
-
-    @observer_bot.command()
-    async def ping(ctx):  # pylint: disable=unused-variable
-        """Standard Ping-Pong latency/is-alive test."""
-        await ctx.send(
-            f"Pong (latency: {observer_bot.latency * 1000:.1f} ms) [`{get_local_machine_name()}`]"
-        )
-
-    @observer_bot.command()
-    async def info(ctx):  # pylint: disable=unused-variable
-        """Query local system information and send it back."""
-        # message = get_info_message()
-        # await ctx.send(message)
-        embed = make_sysinfo_embed()
-        await ctx.send(embed=embed)
-
-    observer_bot.add_cog(SystemResourceObserverCog(observer_bot, channel_id))
-    observer_bot.add_cog(SystemStatsCollectorCog(observer_bot, channel_id))
-
-    LOGGER.info("Start observer bot ...")
-    observer_bot.run(token)
-    LOGGER.info("Quit observer bot.")
-
-
-# ---------------------------------------------------------------------------
-
-
 class ObserverBot(commands.Bot):
-    def __init__(self, *args, channel_id=None, **kwargs):
+    def __init__(
+        self,
+        channel_id: int,
+        *args,
+        name: typing.Optional[str] = None,
+        limits_types: LimitTypesSetType = None,
+        **kwargs,
+    ):
         super().__init__(*args, **kwargs)
         self.channel_id = channel_id
 
+        self.local_machine_name = name or get_name()
+
         self.add_cog(GeneralCommandsCog(self))
-        self.add_cog(SystemResourceObserverCog(self, channel_id))
-        self.add_cog(SystemStatsCollectorCog(self, channel_id))
+        self.add_cog(SystemResourceObserverCog(self, limits_types=limits_types))
+        self.add_cog(SystemStatsCollectorCog(self))
 
     async def on_ready(self):
         LOGGER.info(f"Logged on as {self.user}")
         LOGGER.debug(f"name: {self.user.name}, id: {self.user.id}")
 
-        if self.channel_id is not None:
-            channel = self.get_channel(self.channel_id)
-            LOGGER.info(f"Channel: {channel} {type(channel)} {repr(channel)}")
-            await channel.send(
-                f"Running observer bot on `{get_local_machine_name()}`...\n"
-                f"Type `{self.command_prefix}help` to display available commands."
-            )
+        channel = self.get_channel(self.channel_id)
+        LOGGER.info(f"Channel: {channel} {type(channel)} {repr(channel)}")
+        await channel.send(
+            f"Running observer bot on `{self.local_machine_name}`...\n"
+            f"Type `{self.command_prefix}help` to display available commands."
+        )
 
         await self.change_presence(status=discord.Status.idle)
 
@@ -518,8 +537,35 @@ class ObserverBot(commands.Bot):
 # ---------------------------------------------------------------------------
 
 
-def run_observer(token, channel_id):
-    observer_bot = ObserverBot(channel_id=channel_id, command_prefix=".")
+def run_observer(
+    token: str,
+    channel_id: int,
+    name: typing.Optional[str] = None,
+    limits_types: LimitTypesSetType = None,
+) -> typing.NoReturn:
+    """Starts the observer bot and blocks until finished.
+
+    Parameters
+    ----------
+    token : str
+        bot authentiation token
+    channel_id : int
+        Discord channel id
+    name : typing.Optional[str], optional
+        local machine name, used for filtering, by default None
+    limits_types : LimitTypesSetType or typing.Optional[typing.Set[str]], optional
+        Names of limit types that should be observed,
+        None would mean that only critical limits are used,
+        to disable all, use an empty set, by default None
+    """
+
+    if name:
+        LOGGER.info(f"Set local machine name to: {name}")
+        set_name(name)
+
+    observer_bot = ObserverBot(
+        channel_id, name=name, limits_types=limits_types, command_prefix="."
+    )
     LOGGER.info("Start observer bot ...")
     observer_bot.run(token)
     LOGGER.info("Quit observer bot.")
